<?php
/**
 * Hooks for ControlSpecialVersion extension
 *
 * @file
 * @ingroup Extensions
 */

class ControlSpecialVersionHooks {
	/**
	 * Modify what the Special:Version page displays in the "Software" section
	 */
	public static function onSoftwareInfo( &$software ) {

        global
            $wgControlSpecialVersion_mediawiki_show,
            $wgControlSpecialVersion_mediawiki_show_rcs,
            $wgControlSpecialVersion_php_show,
            $wgControlSpecialVersion_postgres_norevision,
            $wgControlSpecialVersion_mysql_norevision,
            $wgControlSpecialVersion_postgres_rename;

        // MediaWiki items
        $mwkey = '';
        foreach ( preg_grep ( '/\bMediaWiki\b/', array_keys( $software ) ) as $item) {
            $mwkey = $item;
            break;
        }
        if ($mwkey) {
            // Do we show MediaWiki information at all?
            if ( ! $wgControlSpecialVersion_mediawiki_show ) {
                unset( $software[$mwkey] );
            }
            // Do we show the non-version number information?
            else if ( ! $wgControlSpecialVersion_mediawiki_show_rcs ) {
                $software[$mwkey] = preg_replace( '/] .*/', ']', $software[$mwkey] );
            }
        }

        // Do we show PHP information?
        if ( ! $wgControlSpecialVersion_php_show ) {
            // We assume the key always contains a bare 'PHP'
            foreach ( preg_grep ( '/\bPHP\b/', array_keys( $software ) ) as $item) {
                unset( $software[$item] );
            }
        }

        // Postgres items
        $pgkey = '';
        foreach ( preg_grep ( '/\bPostgreSQL\b/', array_keys( $software ) ) as $item) {
            $pgkey = $item;
            break;
        }
        if ($pgkey) {
            // Only show the major Postgres version, not the revision
            if ( $wgControlSpecialVersion_postgres_norevision ) {
                $software[$pgkey] = preg_replace( '/(\d+\.\d+).*/', '$1', $software[$pgkey] );
            }
            // Change "PostgreSQL" to the far better "Postgres"
            if ( $wgControlSpecialVersion_postgres_rename ) {
                $newpgkey = preg_replace( '/PostgreSQL/', 'Postgres', $pgkey );
                $software[$newpgkey] = $software[$pgkey];
                unset( $software[$pgkey] );
            }
        }

        // MySQL items
        $mykey = '';
        foreach ( preg_grep ( '/\bMySQL\b/', array_keys( $software ) ) as $item) {
            $mykey = $item;
            break;
        }
        if ($mykey) {
            // Only show the major MySQL version
            if ( $wgControlSpecialVersion_mysql_norevision ) {
                $software[$mykey] = preg_replace( '/(\d+\.\d+).*/', '$1', $software[$mykey] );
            }
        }

		return true;

	} // end of onSoftwareInfo


	public static function onAPIQuerySiteInfoGeneralInfo( $api, &$software ) {
	/**
	 * Modify what the 'siteinfo' information returns via the API
	 */

        global
            $wgControlSpecialVersion_mediawiki_show,
            $wgControlSpecialVersion_mediawiki_show_rcs,
            $wgControlSpecialVersion_php_show,
            $wgControlSpecialVersion_postgres_norevision;


        // Do we show MediaWiki version
        if ( ! $wgControlSpecialVersion_mediawiki_show ) {
            $software['generator'] = 'MediaWiki';
        }
        // Do we show the non-version number information?
        else if ( ! $wgControlSpecialVersion_mediawiki_show_rcs ) {
            $software['generator'] = preg_replace( '/] .*/', ']', $software['generator'] );
        }

        // Do we show PHP version?
        if ( ! $wgControlSpecialVersion_php_show ) {
            $software['phpversion'] = '';
        }

        // Postgres items
        if ($software['dbtype'] === 'postgres') {
            // Only show the major Postgres version, not the revision
            if ( $wgControlSpecialVersion_postgres_norevision ) {
                $software['dbversion'] = preg_replace( '/(\d+\.\d+).*/', '$1', $software['dbversion'] );
            }
        }

        // MySQL items
        if ($software['dbtype'] === 'mysql') {
            // Only show the major MySQL version, not the revision
            if ( $wgControlSpecialVersion_mysql_norevision ) {
                $software['dbversion'] = preg_replace( '/(\d+\.\d+).*/', '$1', $software['dbversion'] );
            }
        }

        return true;

    } // end of onAPIQuerySiteInfoGeneralInfo

    public static function onExtensionTypes( array &$extTypes ) {
    /**
     * Modify what the Special:Version page displays in the "Extensions" section
     */
        global
            $wgExtensionCredits,
            $wgControlSpecialVersion_extension_show_versions,
            $wgControlSpecialVersion_extensions_hidden,
            $wgControlSpecialVersion_extensions_show_vcs;

        // If we are not changing anything, simply return quickly
        if ( empty( $wgControlSpecialVersion_extensions_hidden )
             && $wgControlSpecialVersion_extension_show_versions) {
            return true;
        }

        foreach ( $wgExtensionCredits as $type => &$typelist) {
            foreach ( $typelist as $extname => &$extinfo ) {

                // Remove the extension if it should be hidden
                if ( array_key_exists( $extname, $wgControlSpecialVersion_extensions_hidden ) ) {
                    unset ( $typelist[$extname] );
                    continue; // No need to worry about the version number now!
                }

                // Prevent the rcs information from being shown
                if ( !$wgControlSpecialVersion_extensions_show_vcs
                     || ! $wgControlSpecialVersion_extension_show_versions ) {
                    // Special:Version tries to tease out some vcs information to display
                    // We want to prevent this, but because it is based on the extension path,
                    // and said path is also use for the license display, we store the license
                    // existence information first:
                    if (! isset( $extinfo['license-name'] ) && isset( $extinfo['path'] ) ) {
                        $extensionPath = dirname( $extinfo['path'] );
                        if (SpecialVersion::getExtLicenseFileName( $extensionPath ) ) {
                            $extinfo['license-name'] = 'License'; // TODO: do this better
                        }
                    }
                    // Now we can safely remove the path, which prevents vcs information from showing
                    unset ( $extinfo['path'] );
                }

                if ( ! $wgControlSpecialVersion_extension_show_versions ) {
                    // Hide version information by removing the version attribute:
                    unset ( $extinfo['version'] );
                }
            }
        }

        return true;

    } // end of onExtensionTypes

} // end of class ControlSpecialVersionHooks
